'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var abstractConnector = require('@web3-react/abstract-connector');
var WalletConnectProvider = _interopDefault(require('@walletconnect/web3-provider'));

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var URI_AVAILABLE = 'URI_AVAILABLE';
var UserRejectedRequestError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(UserRejectedRequestError, _Error);

  function UserRejectedRequestError() {
    var _this;

    _this = _Error.call(this) || this;
    _this.name = _this.constructor.name;
    _this.message = 'The user rejected the request.';
    return _this;
  }

  return UserRejectedRequestError;
}(
/*#__PURE__*/
_wrapNativeSuper(Error));
var WalletConnectConnector =
/*#__PURE__*/
function (_AbstractConnector) {
  _inheritsLoose(WalletConnectConnector, _AbstractConnector);

  function WalletConnectConnector(_ref) {
    var _this2;

    var rpc = _ref.rpc,
        infuraId = _ref.infuraId,
        bridge = _ref.bridge,
        qrcode = _ref.qrcode;
    _this2 = _AbstractConnector.call(this, rpc ? {
      supportedChainIds: Object.keys(rpc).map(function (k) {
        return Number(k);
      })
    } : {}) || this;
    _this2.rpc = rpc;
    _this2.infuraId = infuraId;
    _this2.bridge = bridge;
    _this2.qrcode = qrcode;
    _this2.handleConnect = _this2.handleConnect.bind(_assertThisInitialized(_this2));
    _this2.handleNetworkChanged = _this2.handleNetworkChanged.bind(_assertThisInitialized(_this2));
    _this2.handleChainChanged = _this2.handleChainChanged.bind(_assertThisInitialized(_this2));
    _this2.handleAccountsChanged = _this2.handleAccountsChanged.bind(_assertThisInitialized(_this2));
    _this2.handleClose = _this2.handleClose.bind(_assertThisInitialized(_this2));
    return _this2;
  }

  var _proto = WalletConnectConnector.prototype;

  _proto.handleConnect = function handleConnect() {
    {
      console.log('Logging connect event');
    }
  };

  _proto.handleNetworkChanged = function handleNetworkChanged(networkId) {
    {
      console.log('Handling networkChanged event with payload', networkId);
    } // const chainId = parseInt(networkId)
    // try {
    //   this.validateChainId(chainId)
    //   this.emitUpdate({ chainId })
    // } catch (error) {
    //   this.emitError(error)
    // }

  };

  _proto.handleChainChanged = function handleChainChanged(chainId) {
    {
      console.log('Logging chainChanged event with payload', chainId);
    }

    try {
      this.validateChainId(chainId);
      this.emitUpdate({
        chainId: chainId
      });
    } catch (error) {
      this.emitError(error);
    }
  };

  _proto.handleAccountsChanged = function handleAccountsChanged(accounts) {
    {
      console.log('Handling accountsChanged event with payload', accounts);
    }

    if (accounts.length === 0) {
      this.emitDeactivate();
    } else {
      this.emitUpdate({
        account: accounts[0]
      });
    }
  };

  _proto.handleClose = function handleClose(code, reason) {
    {
      console.log('Logging close event with payload', code, reason);
    } // this.emitDeactivate()

  };

  _proto.activate = function activate() {
    try {
      var _temp3 = function _temp3() {
        return Promise.resolve(provider.enable()["catch"](function (error) {
          // TODO ideally this would be a better check
          if (error.message === 'User closed WalletConnect modal') {
            throw new UserRejectedRequestError();
          }

          throw error;
        })).then(function (accounts) {
          return {
            provider: provider,
            account: accounts[0]
          };
        });
      };

      var _this4 = this;

      _this4.provider = new WalletConnectProvider({
        bridge: _this4.bridge,
        rpc: _this4.rpc === undefined ? undefined : _this4.rpc,
        infuraId: _this4.infuraId,
        qrcode: _this4.qrcode
      });
      var provider = _this4.provider;
      _this4.walletConnector = provider.wc;
      var walletConnector = _this4.walletConnector;
      provider.on('connect', _this4.handleConnect);
      provider.on('networkChanged', _this4.handleNetworkChanged);
      provider.on('chainChanged', _this4.handleChainChanged);
      provider.on('accountsChanged', _this4.handleAccountsChanged);
      provider.on('close', _this4.handleClose); // ensure that the uri is going to be available and emit an event

      var _temp4 = function () {
        if (!walletConnector.connected) {
          return Promise.resolve(walletConnector.createSession({
            chainId: provider.chainId
          })).then(function () {
            _this4.emit(URI_AVAILABLE, walletConnector.uri);
          });
        }
      }();

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getProvider = function getProvider() {
    try {
      var _this6 = this;

      return Promise.resolve(_this6.provider);
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getChainId = function getChainId() {
    try {
      var _this8 = this;

      return Promise.resolve(_this8.provider.send('eth_chainId')).then(function (chainId) {
        _this8.validateChainId(chainId);

        return chainId;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getAccount = function getAccount() {
    try {
      var _this10 = this;

      return Promise.resolve(_this10.provider.send('eth_accounts')).then(function (accounts) {
        return accounts[0];
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.deactivate = function deactivate() {
    var provider = this.provider;
    provider.stop();
    provider.removeListener('connect', this.handleConnect);
    provider.removeListener('networkChanged', this.handleNetworkChanged);
    provider.removeListener('chainChanged', this.handleChainChanged);
    provider.removeListener('accountsChanged', this.handleAccountsChanged);
    provider.removeListener('close', this.handleClose);
    this.walletConnector = undefined;
    this.provider = undefined;
  };

  _proto.close = function close() {
    try {
      var _this12 = this;

      return Promise.resolve(_this12.walletConnector.killSession()).then(function () {
        _this12.emitDeactivate();
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return WalletConnectConnector;
}(abstractConnector.AbstractConnector);

Object.defineProperty(exports, 'UnsupportedChainIdError', {
  enumerable: true,
  get: function () {
    return abstractConnector.UnsupportedChainIdError;
  }
});
exports.URI_AVAILABLE = URI_AVAILABLE;
exports.UserRejectedRequestError = UserRejectedRequestError;
exports.WalletConnectConnector = WalletConnectConnector;
//# sourceMappingURL=walletconnect-connector.cjs.development.js.map
